@@ -6,6 +6,7 @@ from libifstate.sysctl import Sysctl
 from libifstate.parser import Parser
 from libifstate.tc import TC
 
+from pyroute2.netlink.rtnl.ifaddrmsg import IFA_F_PERMANENT
 from pyroute2.netlink.exceptions import NetlinkError
 try:
     from libifstate.wireguard import WireGuard
@@ -27,7 +28,7 @@ import re
 import json
 import errno
 
-__version__ = "1.2.0"
+__version__ = "1.5.0"
 
 
 class IfState():
@@ -36,6 +37,11 @@ class IfState():
         self.links = {}
         self.addresses = {}
         self.ignore = {}
+        self.vrrp = {
+            'links': [],
+            'group': {},
+            'instance': {},
+        }
         self.tables = None
         self.rules = None
         self.sysctl = Sysctl()
@@ -92,7 +98,7 @@ class IfState():
                 raise LinkDuplicate()
             if 'link' in ifstate:
                 self.links[name] = Link(
-                    name, ifstate['link'], ifstate.get('ethtool'))
+                    name, ifstate['link'], ifstate.get('ethtool'), ifstate.get('vrrp'))
             else:
                 self.links[name] = None
 
@@ -101,6 +107,18 @@ class IfState():
             else:
                 self.addresses[name] = None
 
+            if 'vrrp' in ifstate:
+                ktype = ifstate['vrrp']['type']
+                kname = ifstate['vrrp']['name']
+                kstates = ifstate['vrrp']['states']
+                if not kname in self.vrrp[ktype]:
+                    self.vrrp[ktype][kname] = {}
+                for kstate in kstates:
+                    if not kstate in self.vrrp[ktype][kname]:
+                        self.vrrp[ktype][kname][kstate] = []
+                    self.vrrp[ktype][kname][kstate].append(name)
+                self.vrrp['links'].append(name)
+
             if 'sysctl' in ifstate:
                 self.sysctl.add(name, ifstate['sysctl'])
 
@@ -131,13 +149,32 @@ class IfState():
         # add ignore list items
         self.ignore.update(ifstates['ignore'])
 
-    def apply(self):
-        self._apply(True)
+    def apply(self, vrrp_type=None, vrrp_name=None, vrrp_state=None):
+        self._apply(True, vrrp_type, vrrp_name, vrrp_state)
 
-    def check(self):
-        self._apply(False)
+    def check(self, vrrp_type=None, vrrp_name=None, vrrp_state=None):
+        self._apply(False, vrrp_type, vrrp_name, vrrp_state)
+        self._apply(False, [])
+
+    def _apply(self, do_apply, vrrp_type, vrrp_name, vrrp_state):
+        vrrp_ignore = []
+        vrrp_remove = []
+
+        by_vrrp = not None in [
+            vrrp_type, vrrp_name, vrrp_state]
+
+        for ifname, link in self.links.items():
+            if ifname in self.vrrp['links']:
+                if not by_vrrp:
+                    vrrp_ignore.append(ifname)
+                else:
+                    if not link.match_vrrp_select(vrrp_type, vrrp_name):
+                        vrrp_ignore.append(ifname)
+                    elif not vrrp_name in self.vrrp[vrrp_type] or not vrrp_state in self.vrrp[vrrp_type][vrrp_name] or not ifname in self.vrrp[vrrp_type][vrrp_name][vrrp_state]:
+                        vrrp_remove.append(ifname)
+            elif by_vrrp:
+                vrrp_ignore.append(ifname)
 
-    def _apply(self, do_apply):
         self.ipaddr_ignore = set()
         for ip in self.ignore.get('ipaddr', []):
             self.ipaddr_ignore.add(ip_network(ip))
@@ -154,17 +191,27 @@ class IfState():
         for stage in range(2):
             if stage == 0:
                 logger.info("\nconfiguring interface links")
+
+                for ifname in vrrp_remove:
+                    logger.debug('to be removed due to vrrp constraint',
+                                 extra={'iface': ifname})
+                    del self.links[ifname]
             else:
                 logger.info("\nconfiguring interface links (stage 2)")
 
             retry = False
             applied = []
-            while len(applied) < len(self.links):
+            while len(applied) + len(vrrp_ignore) < len(self.links):
                 last = len(applied)
                 for name, link in self.links.items():
                     if name in applied:
                         continue
 
+                    if name in vrrp_ignore:
+                        logger.debug('skipped due to vrrp constraint',
+                                     extra={'iface': name})
+                        continue
+
                     if link is None:
                         logger.debug('skipped due to no link settings',
                                      extra={'iface': name})
@@ -200,6 +247,9 @@ class IfState():
                                     name, err.args[1]))
                     # shutdown physical interfaces
                     else:
+                        if name in vrrp_ignore:
+                            logger.warning('vrrp', extra={
+                                'iface': name, 'style': IfStateLogging.STYLE_OK})
                         if link.get('state') == 'down':
                             logger.warning('orphan', extra={
                                 'iface': name, 'style': IfStateLogging.STYLE_OK})
@@ -220,7 +270,10 @@ class IfState():
             logger.info("\nconfiguring interface traffic control...")
 
             for name, tc in self.tc.items():
-                if tc is None:
+                if name in vrrp_ignore:
+                    logger.debug('skipped due to vrrp constraint',
+                                 extra={'iface': name})
+                elif tc is None:
                     logger.debug('skipped due to no tc settings', extra={
                                  'iface': name})
                 else:
@@ -236,7 +289,10 @@ class IfState():
                     self.addresses[name] = Addresses(name, [])
 
             for name, addresses in self.addresses.items():
-                if addresses is None:
+                if name in vrrp_ignore:
+                    logger.debug('skipped due to vrrp constraint',
+                                 extra={'iface': name})
+                elif addresses is None:
                     logger.debug('skipped due to no address settings', extra={
                                  'iface': name})
                 else:
@@ -254,20 +310,27 @@ class IfState():
         if len(self.wireguard):
             logger.info("\nconfiguring WireGuard...")
             for iface, wireguard in self.wireguard.items():
+                if iface in vrrp_ignore:
+                    logger.debug('skipped due to vrrp constraint',
+                                 extra={'iface': name})
+                    continue
                 wireguard.apply(do_apply)
 
-    def show(self):
-        defaults = deepcopy(Parser._default_ifstates)
+    def show(self, showall=False):
+        if showall:
+            defaults = deepcopy(Parser._default_ifstates)
+        else:
+            defaults = {}
 
-        self.ipaddr_ignore = set()
-        for ip in Parser._default_ifstates.get('ignore').get('ipaddr'):
-            self.ipaddr_ignore.add(ip_network(ip))
+        ipaddr_ignore = []
+        for ip in Parser._default_ifstates['ignore']['ipaddr_builtin']:
+            ipaddr_ignore.append(ip_network(ip))
 
         ifs_links = []
         for ipr_link in ipr.get_links():
             name = ipr_link.get_attr('IFLA_IFNAME')
             # skip links on ignore list
-            if not any(re.match(regex, name) for regex in Parser._default_ifstates['ignore'].get('ifname', [])):
+            if not any(re.match(regex, name) for regex in Parser._default_ifstates['ignore']['ifname_builtin']):
                 ifs_link = {
                     'name': name,
                     'addresses': [],
@@ -277,34 +340,55 @@ class IfState():
                 }
 
                 for addr in ipr.get_addr(index=ipr_link['index']):
-                    ip = ip_interface(addr.get_attr(
-                        'IFA_ADDRESS') + '/' + str(addr['prefixlen']))
-                    if not any(ip in net for net in self.ipaddr_ignore):
-                        ifs_link['addresses'].append(ip.with_prefixlen)
+                    if addr['flags'] & IFA_F_PERMANENT == IFA_F_PERMANENT:
+                        ip = ip_interface(addr.get_attr(
+                            'IFA_ADDRESS') + '/' + str(addr['prefixlen']))
+                        if not any(ip in net for net in ipaddr_ignore):
+                            ifs_link['addresses'].append(ip.with_prefixlen)
 
                 info = ipr_link.get_attr('IFLA_LINKINFO')
-                if info is not None:
+                if info is None:
+                    kind = None
+                else:
                     kind = info.get_attr('IFLA_INFO_KIND')
+                if kind is not None:
                     ifs_link['link']['kind'] = kind
 
                     data = info.get_attr('IFLA_INFO_DATA')
                     # unsupported link type, fallback to raw encoding
-                    if type(data) == str:
-                        ifs_link['link']["info_data"] = data
-                    elif data is not None:
+                    if data is not None and type(data) != str:
                         for k, v in data['attrs']:
-                            ifs_link['link'][ipr_link.nla2name(k)] = v
+                            if k != "UNKNOWN" and type(data) in [str, int] and v != 0:
+                                ifs_link['link'][ipr_link.nla2name(k)] = v
                 else:
                     ifs_link['link']['kind'] = 'physical'
                     addr = ipr_link.get_attr('IFLA_ADDRESS')
                     if not addr is None:
                         ifs_link['link']['address'] = addr
+                    permaddr = ipr.get_permaddr(name)
+                    if not permaddr is None:
+                        if addr is None:
+                            ifs_link['link']['addr'] = permaddr
+                        elif addr != permaddr:
+                            ifs_link['link']['permaddr'] = permaddr
+                    businfo = ipr.get_businfo(name)
+                    if not businfo is None:
+                        ifs_link['link']['businfo'] = businfo
+
+                master = ipr_link.get_attr('IFLA_MASTER')
+                if master is not None:
+                    ifs_link['link']['master'] = ipr.get_ifname_by_index(
+                        master)
+
+                mtu = ipr_link.get_attr('IFLA_MTU')
+                if not mtu is None and not mtu in [1500, 65536]:
+                    ifs_link['link']['mtu'] = mtu
 
                 ifs_links.append(ifs_link)
 
         routing = {
-            'routes': Tables().show_routes(Parser._default_ifstates['ignore']['routes']),
-            'rules': Rules().show_rules(Parser._default_ifstates['ignore']['rules']),
+            'routes': Tables().show_routes(Parser._default_ifstates['ignore']['routes_builtin']),
+            'rules': Rules().show_rules(Parser._default_ifstates['ignore']['rules_builtin']),
         }
 
         return {**defaults, **{'interfaces': ifs_links, 'routing': routing}}
