@@ -3,6 +3,7 @@ from libifstate.exception import RouteDupblicate
 from ipaddress import ip_address, ip_network
 from pyroute2.netlink.exceptions import NetlinkError
 from pyroute2.netlink.rtnl.fibmsg import FR_ACT_VALUES
+from pyroute2.netlink.rtnl import rt_type
 import collections.abc
 from glob import glob
 import os
@@ -12,7 +13,7 @@ import socket
 from socket import AF_INET, AF_INET6
 
 
-def route_matches(r1, r2, fields=('dst', 'metric', 'proto'), verbose=False, indent=""):
+def route_matches(r1, r2, fields=('dst', 'priority', 'proto'), verbose=False, indent=""):
     return _matches(r1, r2, fields, verbose, indent)
 
 
@@ -94,19 +95,30 @@ class Tables(collections.abc.Mapping):
     def add(self, route):
         rt = {
             'table': RTLookups.tables.lookup_id(route.get('table', 254)),
+            'type': route.get('type', 'unicast'),
             'dst': ip_network(route['to']).with_prefixlen,
             'scope': RTLookups.scopes.lookup_id(route.get('scope', 0)),
             'proto': RTLookups.protos.lookup_id(route.get('proto', 3)),
             'realm': RTLookups.realms.lookup_id(route.get('realm', 0)),
             'tos': route.get('tos', 0),
+            'priority': 0
         }
 
+        if type(rt['type']) == str:
+            rt['type'] = rt_type[rt['type']]
+
         if 'dev' in route:
             rt['oif'] = route['dev']
 
         if 'via' in route:
             rt['gateway'] = str(ip_address(route['via']))
 
+        if 'src' in route:
+            rt['prefsrc'] = route['src']
+
+        if 'preference' in route:
+            rt['priority'] = route['preference']
+
         if not rt['table'] in self.tables:
             self.tables[rt['table']] = []
         self.tables[rt['table']].append(rt)
@@ -138,9 +150,11 @@ class Tables(collections.abc.Mapping):
 
             rt = {
                 'to': dst,
-                'table': RTLookups.tables.lookup_str(table),
             }
 
+            if table != 254:
+                rt['table'] = RTLookups.tables.lookup_str(table),
+
             dev = route.get_attr('RTA_OIF')
             if dev:
                 link = next(iter(ipr.get_links(dev)), None)
@@ -149,6 +163,10 @@ class Tables(collections.abc.Mapping):
                 else:
                     rt['dev'] = dev
 
+            via = route.get_attr('RTA_GATEWAY')
+            if via:
+                rt['via'] = via
+
             realm = route.get_attr('RTA_FLOW')
             if realm:
                 rt['realm'] = RTLookups.realms.lookup_str(realm)
@@ -162,6 +180,18 @@ class Tables(collections.abc.Mapping):
             if route['tos'] != 0:
                 rt['tos'] = route['tos']
 
+            src = route.get_attr('RTA_PREFSRC')
+            if src:
+                rt['src'] = src
+
+            rtype = route['type']
+            if rtype != 1:
+                rt['type'] = rt_type[rtype]
+
+            priority = route.get_attr('RTA_PRIORITY')
+            if priority:
+                rt['preference'] = priority
+
             routes.append(rt)
 
         return routes
@@ -183,12 +213,14 @@ class Tables(collections.abc.Mapping):
 
             rt = {
                 'table': route.get_attr('RTA_TABLE'),
+                'type': route['type'],
                 'dst': dst,
                 'oif': route.get_attr('RTA_OIF'),
                 'scope': route['scope'],
                 'proto': route['proto'],
                 'realm': route.get_attr('RTA_FLOW', 0),
                 'tos': route['tos'],
+                'priority': 0
             }
 
             gateway = route.get_attr('RTA_GATEWAY')
@@ -203,6 +235,14 @@ class Tables(collections.abc.Mapping):
             if not pref is None:
                 rt['pref'] = pref
 
+            prefsrc = route.get_attr('RTA_PREFSRC')
+            if not prefsrc is None:
+                rt['prefsrc'] = prefsrc
+
+            priority = route.get_attr('RTA_PRIORITY')
+            if not priority is None:
+                rt['priority'] = priority
+
             routes.append(rt)
         return routes
 
@@ -398,18 +438,20 @@ class Rules():
                 "FR_ACT_NAT": "nat",
             }.get(rule['action'], rule['action'])
 
+            if rule['action'] == "to_tbl":
+                del rule['action']
 
-            if 'protocol' in rule:
+            if 'protocol' in rule and rule['protocol'] > 0:
                 rule['proto'] = rule['protocol']
                 del rule['protocol']
 
-            if rule['src_len'] > 0:
+            if 'src_len' in rule and rule['src_len'] > 0:
                 rule['from'] = ip_network(
                     '{}/{}'.format(rule['src'], rule['src_len'])).with_prefixlen
-            del rule['src']
-            del rule['src_len']
-            del rule['family']
-            del rule['tos']
+
+            for key in ['src', 'src_len', 'tos', 'protocol']:
+                if key in rule:
+                    del rule[key]
 
             rules.append(rule)
 
